/*
 * Copyright (C) 2003 Jean-Philippe Hergott <hergott@free.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more detailvins.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <errno.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#define GLADE_HOOKUP_OBJECT(component,widget,name) \
	g_object_set_data_full (G_OBJECT (component), name, \
    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)

#define GLADE_HOOKUP_OBJECT_NO_REF(component,widget,name) \
	g_object_set_data (G_OBJECT (component), name, widget)

void
xcave_CreateCasier(GtkWidget *window, GList *vin_list_emplacement)
{

	GtkWidget   *hbox_rack;
	GtkWidget   *scrolledwindow_rack;
	GtkWidget   *viewport_rack;
	GtkWidget   *frameCasier[Max_Casier];
    GtkWidget    *tableCasier[Max_Casier];
	GtkWidget    *labelCasierUnit[Max_Casier][Max_Casier_Row][Max_Casier_Column];
	GtkWidget    *checkbutton[Max_Casier][Max_Casier_Row][Max_Casier_Column];
	GtkTooltips  *tooltips;
	GList        *tmp_list_emp;
	GList        *tmp_list_casier;

	int           NumCasier;

#if (defined DEBUG_LEVEL_1)
	printf("xcave_CreateCasier:B:\n");
#endif

	scrolledwindow_rack = lookup_widget (window, "scrolledwindow_rack");

	tooltips = gtk_tooltips_new ();

	viewport_rack = gtk_viewport_new (NULL, NULL);
	gtk_widget_show (viewport_rack);
	gtk_container_add (GTK_CONTAINER (scrolledwindow_rack), viewport_rack);
	GLADE_HOOKUP_OBJECT (window, viewport_rack, "viewport_rack");
	
	hbox_rack = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox_rack);
	gtk_container_add (GTK_CONTAINER (viewport_rack), hbox_rack);
	GLADE_HOOKUP_OBJECT (window, hbox_rack, "hbox_rack");

	for (NumCasier=0; NumCasier < CasierNb; NumCasier++) {
		char   buf[64];
		char   lab[32];

		/* 
		 * create as much as frame than racks number 
		 */
		sprintf(buf,"frameCasier_rack%d",NumCasier+1);
		sprintf(lab,_("racks %d"),NumCasier+1);
		
		frameCasier[NumCasier] = gtk_frame_new (lab);
		gtk_widget_set_name (frameCasier[NumCasier], buf);
		gtk_widget_show (frameCasier[NumCasier]);
		gtk_box_pack_start (GTK_BOX (hbox_rack), frameCasier[NumCasier], TRUE, TRUE, 0);
		GLADE_HOOKUP_OBJECT (window, frameCasier[NumCasier], buf);

		/* 
		 * For each frame create a table 
		 */
		sprintf(buf,"tableCasier_rack%d",NumCasier+1);
		
		tableCasier[NumCasier] = gtk_table_new (CasierNbRow, CasierNbColumn, TRUE);
		gtk_widget_set_name (tableCasier[NumCasier], buf);
		gtk_widget_show (tableCasier[NumCasier]);
		gtk_container_add (GTK_CONTAINER (frameCasier[NumCasier]), tableCasier[NumCasier]);
		GLADE_HOOKUP_OBJECT (window, tableCasier[NumCasier], buf); 
	}

	/*
	 * 
	 */
	for (NumCasier=0; NumCasier < CasierNb; NumCasier++) {
	int    NumRow;
	
		// label ligne
		for (NumRow=1; NumRow < CasierNbRow + 1; NumRow++) {
			int    NumCol;
		
			// label colonne
			for (NumCol=1; NumCol < CasierNbColumn + 1; NumCol++) {
				char			buf[64];
				Emplacement		emp;
				int				trouve = 0;
				Emplacement		tmpEmplacement;
				ptrCasier		tmp_casier;
				ptrVin			tmp_vin;
		
				sprintf(buf,"checkbutton%d-%d-%d",NumCasier,NumRow,NumCol);
			
				memset(&emp,'\0',sizeof(Emplacement));
				emp.Casier = NumCasier;
				emp.Row    = NumRow;
				emp.Column = NumCol;
		
				/*
				 * recherche de l'emplacement de la(es) bouteilles selectionnees
				 * pour autoriser l'utilisateur a deplacer les bouteilles dans le 
				 * stock l'emplacement n'est pas fige
				 */
				for(tmp_list_emp = vin_list_emplacement; 
					tmp_list_emp != NULL ; 
					tmp_list_emp = tmp_list_emp->next) {
						memcpy(&tmpEmplacement,
								tmp_list_emp->data,
								sizeof(Emplacement));
						if (  NumCasier == tmpEmplacement.Casier
						   && NumRow    == tmpEmplacement.Row
						   && NumCol    == tmpEmplacement.Column ) { trouve = 1 ; break; }
				}
			
				/*
				 * recherche si l'emplacement est occupe ou non 
				 */
				for(tmp_list_casier = list_casier;
					tmp_list_casier != NULL && trouve == 0 ; 
					tmp_list_casier = tmp_list_casier->next)
						if ( ! memcmp(&emp, tmp_list_casier->data, 3 * sizeof(int)))
							break;
						
				if (tmp_list_casier != NULL) {		
					tmp_casier	= g_malloc(sizeof(Casier));
					tmp_vin		= g_malloc(sizeof(Vin));	
					memcpy(tmp_casier, tmp_list_casier->data, sizeof(Casier));
					memcpy(tmp_vin, tmp_casier->ptr_list_stock_data, sizeof(Vin));
				}
						
				if (tmp_list_casier == NULL || trouve == 1) 
					checkbutton[NumCasier][NumRow][NumCol] = gtk_check_button_new();
				else {
					checkbutton[NumCasier][NumRow][NumCol] = gtk_check_button_new ();
					GtkWidget *image;
					if (strcasecmp(tmp_vin->color, _("RED")) == 0)
						image = create_pixmap (window, "xcave_red.png");
					else
						if (strcasecmp(tmp_vin->color, _("ROSE")) == 0)
							image = create_pixmap (window, "xcave_rose.png");
						else
							image = create_pixmap (window, "xcave_white.png");
					gtk_widget_show (image);
					gtk_container_add (GTK_CONTAINER (checkbutton[NumCasier][NumRow][NumCol]), image);
				}
			
				gtk_widget_set_name (checkbutton[NumCasier][NumRow][NumCol], buf);
				GLADE_HOOKUP_OBJECT (window, checkbutton[NumCasier][NumRow][NumCol], buf);
		
#if (defined DEBUG_LEVEL_1)
				printf("xcave_CreateCasier:checkbutton:%s:%p:\n",buf,(void *)checkbutton[NumCasier][NumRow][NumCol]);
#endif
				gtk_widget_show (checkbutton[NumCasier][NumRow][NumCol]);
				gtk_table_attach (GTK_TABLE (tableCasier[NumCasier]), 
								  checkbutton[NumCasier][NumRow][NumCol], 
								  NumCol,
								  NumCol + 1,
								  NumRow,
								  NumRow + 1,
								  (GtkAttachOptions) (GTK_FILL),
								  (GtkAttachOptions) (0), 0, 0);
				GTK_WIDGET_UNSET_FLAGS (checkbutton[NumCasier][NumRow][NumCol], GTK_CAN_FOCUS);
		
				/*
				 * bouteille dans l'emplacement 
				 * emplacement non modifiable
				 */
				if (tmp_list_casier != NULL && trouve == 0) {
					gtk_button_set_relief (GTK_BUTTON (checkbutton[NumCasier][NumRow][NumCol]), GTK_RELIEF_NONE);
					gtk_widget_set_sensitive (checkbutton[NumCasier][NumRow][NumCol], TRUE);
					gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (checkbutton[NumCasier][NumRow][NumCol]), FALSE);
					gtk_tooltips_set_tip (tooltips,checkbutton[NumCasier][NumRow][NumCol] , tmp_casier->Nom, NULL);
				}
				/*
				 * bouteille dans l'emplacement
				 * emplacement modifiable ==> deplacement du vin autorise
				 */
				if (tmp_list_casier != NULL && trouve == 1) {
					gtk_button_set_relief (GTK_BUTTON (checkbutton[NumCasier][NumRow][NumCol]), GTK_RELIEF_NONE);
					gtk_widget_set_sensitive (checkbutton[NumCasier][NumRow][NumCol], TRUE);
					gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (checkbutton[NumCasier][NumRow][NumCol]), TRUE);
					gtk_tooltips_set_tip (tooltips,checkbutton[NumCasier][NumRow][NumCol] , _(tmpEmplacement.Nom), NULL);
					gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton[NumCasier][NumRow][NumCol]), TRUE);
				  }
				/*
				 * for window_rack need to arm a signal
				 */
				if (window == window_rack) {
					g_signal_connect ((gpointer) checkbutton[NumCasier][NumRow][NumCol], "toggled",
										G_CALLBACK (on_window_rack_toggledbutton_toggled),
										NULL);
				}
		
				/*
				 * set widget name
				 */
				char   lab[14];
		
				sprintf(buf,"labelCasierUnit_rack%d-%d-%d",NumCasier,NumRow+1,NumCol+1);
				memset(lab,'\0',14);
				if ( NumRow == 0 && NumCol != 0) sprintf(lab,"%d",NumCol);
				if ( NumCol == 0 && NumRow != 0) sprintf(lab,"%d",NumRow);
			
				labelCasierUnit[NumCasier][NumRow][NumCol] = gtk_label_new (lab);
				gtk_widget_set_name (labelCasierUnit[NumCasier][NumRow][NumCol], buf);
				GLADE_HOOKUP_OBJECT (window, labelCasierUnit[NumCasier][NumRow][NumCol], buf); 
				gtk_widget_show (labelCasierUnit[NumCasier][NumRow][NumCol]);
				gtk_table_attach (GTK_TABLE (tableCasier[NumCasier]),
								  labelCasierUnit[NumCasier][NumRow][NumCol],
								  NumCol,
								  NumCol + 1,
								  NumRow,
								  NumRow + 1,
								 (GtkAttachOptions) (GTK_FILL),
								 (GtkAttachOptions) (0), 0, 0);
				gtk_misc_set_alignment (GTK_MISC (labelCasierUnit[NumCasier][NumRow][NumCol]), 0, 0.5);
				
				if (tmp_list_casier != NULL && window_detail_origine == BUTTON_RACK) {		
					/*
					 * keep track of the widget name in list_casier to help to 
					 * to retrieve wine data if selection is made thru 
					 * window_rack
					 */
					tmp_casier->ptr_widget_name = checkbutton[NumCasier][NumRow][NumCol];
					memcpy(tmp_list_casier->data, tmp_casier, sizeof(Casier));
					g_free(tmp_vin);
				}
			}
		}
	
	}
#if (defined DEBUG_LEVEL_1)
	printf("xcave_CreateCasier:E:\n");
#endif
}

GList *
xcave_TraitementCasier(char *VinNom)
{

  GtkWidget     *checkbutton;
  int            NumCasier;
  Emplacement 	*ptemp;
  GList         *vin_list_emplacement = NULL;;

#if (defined DEBUG_LEVEL_1)
	printf("xcave_TraitementCasier:B:\n");
#endif

  for (NumCasier=0; NumCasier < CasierNb; NumCasier++) {
    int    NumRow;

    // ligne
    for (NumRow=1; NumRow < CasierNbRow + 1; NumRow++) {
        int    NumCol;

		// colonne
        for (NumCol=1; NumCol < CasierNbColumn + 1; NumCol++) {
            char          buf[64];

			sprintf(buf,"checkbutton%d-%d-%d",NumCasier,NumRow,NumCol);
			checkbutton = (GtkWidget *) lookup_widget (window_detail, buf);

			if ((gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON (checkbutton))) == 1) {
	
				ptemp = malloc (sizeof(Emplacement));
				memset(ptemp,'\0',sizeof(Emplacement));
				ptemp->Casier = NumCasier;
				ptemp->Row    = NumRow;
				ptemp->Column = NumCol;
				strcpy(ptemp->Nom,VinNom);
		
				vin_list_emplacement   = g_list_append(vin_list_emplacement  ,ptemp);
			}
		}
    }
  }
#if (defined DEBUG_LEVEL_1)
	printf("xcave_TraitementCasier:E:\n");
#endif
  return(vin_list_emplacement);
}

void
on_xcave_window_button_rack_clicked    (GtkButton       *button,
                                        gpointer         user_data)
{

#if (defined DEBUG_LEVEL_1)
	printf("on_xcave_window_button_rack_clicked:B:\n");
#endif
	/* 
	 * desactive window_xcave
	 */
	gtk_widget_set_sensitive(window_xcave, FALSE);

	window_rack = create_window_rack();
	window_detail_origine = BUTTON_RACK;
	xcave_CreateCasier(window_rack, NULL);

	g_signal_connect ((gpointer) window_rack, "check_resize",
                    G_CALLBACK (on_window_rack_check_resize),
                    NULL);
	/*
	 * redimensionnement de la fenetre
	 */
	if ( strlen(WindowsRackHeight) != 0 && strlen (WindowsRackWidth) != 0 ) {
	   gtk_window_set_default_size (GTK_WINDOW (window_rack), 
									atoi(WindowsRackWidth), 
									atoi(WindowsRackHeight));
	}
	else
		gtk_widget_set_size_request (window_rack, 160, 350);


	gtk_widget_show (window_rack);

#if (defined DEBUG_LEVEL_1)
	printf("on_xcave_window_button_rack_clicked:E:\n");
#endif
}

void
on_window_rack_destroy               (GtkObject       *object,
                                        gpointer         user_data)
{
#if (defined DEBUG_LEVEL_1)
	printf("on_window_rack_destroy:B:\n");
#endif
	gtk_object_destroy (GTK_OBJECT (window_rack));

	/* 
     * reactive window_xcave
     */
	gtk_widget_set_sensitive(window_xcave, TRUE);
#if (defined DEBUG_LEVEL_1)
	printf("on_window_rack_destroy:E:\n");
#endif
}

void
on_window_rack_button_close_clicked    (GtkButton       *button,
                                        gpointer         user_data)
{
#if (defined DEBUG_LEVEL_1)
	printf("on_window_rack_button_close_clicked:B:\n");
#endif
	gtk_object_destroy (GTK_OBJECT (window_rack));

	/* 
     * reactive window_xcave
     */
	gtk_widget_set_sensitive(window_xcave, TRUE);
#if (defined DEBUG_LEVEL_1)
	printf("on_window_rack_button_close_clicked:E:\n");
#endif
}

void
on_window_rack_toggledbutton_toggled	(GtkToggleButton	*toggledbutton,
										 gpointer         user_data)
{
#if (defined DEBUG_LEVEL_1)
	printf("on_window_rack_toggledbutton_toggled:B:\n");
#endif

	GList		*tmp_list_casier;
	ptrCasier	tmp_casier;
	ptrVin		tmp_vin;
	
	if (gtk_toggle_button_get_active(toggledbutton) == FALSE)
		return;
	
	/* recherche si l'emplacement est occupe ou non */
	tmp_casier	= g_malloc(sizeof(Casier));
	tmp_vin		= g_malloc(sizeof(Vin));	

	for(tmp_list_casier = list_casier;
		tmp_list_casier != NULL; 
		tmp_list_casier = tmp_list_casier->next) {
			memcpy(tmp_casier, tmp_list_casier->data, sizeof(Casier));
			if ((void *)tmp_casier->ptr_widget_name == (void *)toggledbutton) {
				memcpy(tmp_vin, tmp_casier->ptr_list_stock_data, sizeof(Vin));
				break;
			}
	}

#if (defined DEBUG_LEVEL_1)
	if (tmp_list_casier != NULL) 
		printf("on_window_rack_toggledbutton_toggled:%s:%d:\n",(gchar *)tmp_vin->Nom,tmp_vin->vintage);
#endif

	/* 
	 * desactive window_rack
	 */
	gtk_widget_set_sensitive(window_rack, FALSE);
	
	if (tmp_list_casier == NULL) {
		window_detail_origine = BUTTON_RACK_ADD;
		on_xcave_window_add_button_clicked(NULL,NULL);
		/* I could activate the button 
		   retrouver dans l'emplacement avec le triplet 
		   rack-line-colum. 
		   NOT YET IN THE POCKET */
		GtkWidget     *checkbutton;
		int            NumCasier;
		for (NumCasier=0; NumCasier < CasierNb; NumCasier++) {
			int    NumRow;
			
			// ligne
			for (NumRow=1; NumRow < CasierNbRow + 1; NumRow++) {
				int    NumCol;
			
				// colonne
				for (NumCol=1; NumCol < CasierNbColumn + 1; NumCol++) {
					char          buf[64];
			
					sprintf(buf,"checkbutton%d-%d-%d",NumCasier,NumRow,NumCol);
					checkbutton = (GtkWidget *) lookup_widget (window_rack, buf);
			
					if ((gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON (checkbutton))) == 1) {
						checkbutton = (GtkWidget *) lookup_widget (window_detail, buf);
						gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON (checkbutton), TRUE); /* JPH */
						break;
					}
				} /* endfor (NumCol=1; NumCol < CasierNbColumn + 1; NumCol++) */
			} /* endfor (NumRow=1; NumRow < CasierNbRow + 1; NumRow++) */
		} /* endfor (NumCasier=0; NumCasier < CasierNb; NumCasier++) */
	
	}
	else {
		window_detail_origine = BUTTON_RACK_MODIF;
		on_xcave_window_display_modif(tmp_vin);
	}

	g_free(tmp_casier);
	g_free(tmp_vin);

#if (defined DEBUG_LEVEL_1)
	printf("on_window_rack_toggledbutton_toggled:E:\n");
#endif
}

ptrVin 
xcave_id_selected_toggledbutton	(void)
{

	GtkWidget	*toggledbutton;

	GList		*tmp_list_casier;
	ptrCasier	tmp_casier;

	int			NumCasier;

#if (defined DEBUG_LEVEL_1)
	printf("xcave_id_selected_toggledbutton:B:\n");
#endif

	for (NumCasier=0; NumCasier < CasierNb; NumCasier++) {
		int			NumRow;

		// ligne
		for (NumRow=1; NumRow < CasierNbRow + 1; NumRow++) {
			int			NumCol;

			// colonne
			for (NumCol=1; NumCol < CasierNbColumn + 1; NumCol++) {
				char          buf[64];

				// sprintf(buf,"checkbutton_rack%d-%d-%d",NumCasier,NumRow,NumCol);
				sprintf(buf,"checkbutton%d-%d-%d",NumCasier,NumRow,NumCol);
				toggledbutton = (GtkWidget *) lookup_widget (window_rack, buf);
				if ((gtk_toggle_button_get_active( 
						GTK_TOGGLE_BUTTON (toggledbutton))) == TRUE) {
#if (defined DEBUG_LEVEL_1)
						printf("xcave_id_selected_toggledbutton:%d:%d:%d:\n",NumCasier,NumRow,NumCol);
#endif
						gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggledbutton),FALSE);
						
						/* get associated datas */
						tmp_casier	= g_malloc(sizeof(Casier));
					
						for(tmp_list_casier = list_casier;
							tmp_list_casier != NULL; 
							tmp_list_casier = tmp_list_casier->next) {
								memcpy(tmp_casier, tmp_list_casier->data, sizeof(Casier));
								/* retrouver le bon GList */
								if ((void *)tmp_casier->ptr_widget_name == (void *)toggledbutton) {
									break;
									}
						} /* endfor(tmp_list_casier = list_casier */
						break;
				} /* endif ((gtk_toggle_button_get_active( */
			} /* endfor (NumCol=1; NumCol < CasierNbColumn + 1; NumCol++) */
		} /* endfor (NumRow=1; NumRow < CasierNbRow + 1; NumRow++)*/
	} /* endfor (NumCasier=0; NumCasier < CasierNb; NumCasier++) */

#if (defined DEBUG_LEVEL_1)
	printf("xcave_id_selected_toggledbutton:E:\n");
#endif
	return(tmp_casier->ptr_list_stock_data);

}
