/*
 * Copyright (C) 2003 Jean-Philippe Hergott <hergott@free.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include "sys/wait.h"

#include "callbacks.h"
#include "interface.h"
#include "support.h"


void
on_xcave_window_button_print_clicked                       (GtkButton       *button,
                                        gpointer         user_data)
{
#if (defined DEBUG_LEVEL_1)
	printf("on_xcave_window_button_print_clicked:B:\n");
#endif
    // initialisation
    memset(FileLocation,'\0',sizeof(FileLocation));
    strcpy(FileLocation,getenv("HOME"));
    strncat(FileLocation,"/xcave.txt",10);

	/* 
	 * desactive window_xcave
	 */
	gtk_widget_set_sensitive(window_xcave, FALSE);

	window_print = create_window_print();

	flagprint = lpr;
	gtk_entry_set_text(GTK_ENTRY(lookup_widget (window_print, "entry_printer")),"lpr");

	gtk_widget_show (window_print);

	flagrecord = all;

#if (defined DEBUG_LEVEL_1)
	printf("on_xcave_window_button_print_clicked:E:\n");
#endif
}


void
on_window_print_destroy    (GtkObject       *object,
							gpointer         user_data)
{

   gtk_object_destroy (GTK_OBJECT (window_print));

   /* 
    * reactive window_xcave
    */
   gtk_widget_set_sensitive(window_xcave, TRUE);
}


void
on_window_print_button_cancel_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_object_destroy (GTK_OBJECT (window_print));

   /* 
    * reactive window_xcave
    */
   gtk_widget_set_sensitive(window_xcave, TRUE);
}


void
on_window_print_button_ok_clicked(GtkButton       *button,
                                  gpointer         user_data)
{

  GList            *tmp_list_stock;
  GList            *tmp_list_emplacement;
  Vin              *tmpVin;
  ptrEmplacement    tmpEmplacement = NULL;

  pid_t   pid = 1;
  char    bprinter[80];
  char    tprinter[80];
  int     fd;
  char    print_buffer[80];

#if (defined DEBUG_LEVEL_1)
	printf("on_window_print_button_ok_clicked:B:\n");
#endif

  /* where shall we write */

  memset(tprinter,'\0',sizeof(tprinter));
  memset(bprinter,'\0',sizeof(bprinter));
  memset(print_buffer,'\0',sizeof(print_buffer));

  if ( flagprint == lpr ) {
     strcpy(bprinter,"/tmp/.xcave.lst");
     strcpy(tprinter,gtk_entry_get_text(GTK_ENTRY(lookup_widget (window_print, "entry_printer"))));
  } else 
     strcpy(bprinter,gtk_entry_get_text(GTK_ENTRY(lookup_widget (window_print, "entry_printer"))));
       
  /* open file for writing */
  fd = open (bprinter,O_RDWR|O_CREAT|O_TRUNC,S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
  if (fd<0) {
     g_warning ("file error");
  }

  /*
   * Impression de la totalite du stock
   */
  if ( flagrecord == all ) {
	 tmpVin = malloc (sizeof (Vin));
  
     /* c'est parti pour l'entete */
     write(fd,print_ligne_separ,strlen(print_ligne_separ)); 
     sprintf(print_buffer,"| %-11s| %-18s| %-31s| %-4.4s| %-3.3s|\n",_("Region"),
		                                                     _("Appellation"),
								     _("Domain"),
								     _("v.year"),
								     _("qty."));
     write(fd,print_buffer,strlen(print_entete)); 
     write(fd,print_ligne_separ,strlen(print_ligne_separ)); 

     /* et maintenant le fichier Stocks */
     list_stock = g_list_sort(list_stock,(GCompareFunc)cmp_region);
     for(tmp_list_stock = list_stock; tmp_list_stock != NULL; tmp_list_stock = tmp_list_stock->next) {
        memset(tmpVin,'\0',sizeof(Vin));
        memcpy(tmpVin,tmp_list_stock->data,sizeof(Vin));
	
        memset(print_buffer,'\0',sizeof(print_buffer));
  
        sprintf(print_buffer,"|%-12.12s|%-19.19s|%-32.32s| %4.4d| %3.d|\n",tmpVin->Region,
			                                                    tmpVin->Appellation,
									    tmpVin->Nom,
									    tmpVin->vintage,
									    tmpVin->Qte);
        if (tmpVin->Qte != 0) write(fd,print_buffer,sizeof(print_buffer)-1); 

     } /* END for */

     write(fd,print_ligne_separ,strlen(print_ligne_separ)); 

     g_free(tmpVin);
	  
  } 
  else {

	tmpVin = xcave_id_selected_treelist_row();
    /*
     * Impression de l'element selectionn dans le stock
     */
	if (tmpVin != NULL) {
		write(fd,print_ligne_separ,strlen(print_ligne_separ)); 
		 
		sprintf(print_buffer,_("| Domain: %-48s Vintage Year: %d |\n"),tmpVin->Nom,tmpVin->vintage);
		write(fd,print_buffer,strlen(print_buffer)); 
		memset(print_buffer,'\0',sizeof(print_buffer));
	
		write(fd,print_ligne_separ,strlen(print_ligne_separ)); 
	
		/*
		 * General
		 */
		sprintf(print_buffer,_("| Country: %-24s         %32.32s |\n"),tmpVin->country,tmpVin->Region);
		write(fd,print_buffer,strlen(print_buffer)); 
		memset(print_buffer,'\0',sizeof(print_buffer));
		sprintf(print_buffer,_("| Appellation: %-61s |\n"),tmpVin->Appellation);
		write(fd,print_buffer,strlen(print_buffer)); 
		memset(print_buffer,'\0',sizeof(print_buffer));
		sprintf(print_buffer,_("| Variety: %-65s |\n"),tmpVin->Cepage);
		write(fd,print_buffer,strlen(print_buffer)); 
		memset(print_buffer,'\0',sizeof(print_buffer));
		sprintf(print_buffer,_("| Color: %-32s                                    |\n"),tmpVin->color);
		write(fd,print_buffer,strlen(print_buffer)); 
		memset(print_buffer,'\0',sizeof(print_buffer));
	
		/*
		 * Stock
		 */
		write(fd,print_ligne_separ,strlen(print_ligne_separ)); 
		sprintf(print_buffer,_("| Stock  %-52s Quantity:  %3.d |\n")," ",tmpVin->Qte);
		write(fd,print_buffer,strlen(print_buffer)); 
	
		tmpEmplacement = malloc (sizeof(Emplacement));
		for(tmp_list_emplacement = tmpVin->list_emplacement; 
			tmp_list_emplacement != NULL; 
			tmp_list_emplacement = tmp_list_emplacement->next) {
	
			memset(tmpEmplacement,'\0',sizeof(Emplacement));
			memcpy(tmpEmplacement,tmp_list_emplacement->data,sizeof(Emplacement));
	
			memset(print_buffer,'\0',sizeof(print_buffer));
			sprintf(print_buffer,_("| Rack: %2.d   Row: %2.d   Column: %2.d %-43s|\n"),
							     tmpEmplacement->Casier+1,
								 tmpEmplacement->Row,
								 tmpEmplacement->Column,
								 " ");
			write(fd,print_buffer,strlen(print_buffer)); 
		}
		free(tmpEmplacement);
		memset(print_buffer,'\0',sizeof(print_buffer));
	
		/*
		 * Achat
		 */
		write(fd,print_ligne_separ,strlen(print_ligne_separ)); 
		sprintf(print_buffer,_("| Purchase  %-64s |\n")," ");
		write(fd,print_buffer,strlen(print_buffer)); 
	
		/*
		 * Consommation
		 */
		write(fd,print_ligne_separ,strlen(print_ligne_separ)); 
		sprintf(print_buffer,_("| Drinking  %-64s |\n")," ");
		write(fd,print_buffer,strlen(print_buffer)); 
		 
		/*
		 * Comment
		 */
		write(fd,print_ligne_separ,strlen(print_ligne_separ)); 
		{
			int i = 0;
			for (i = 0 ; i < strlen(tmpVin->Commentaire);) {
				int j = 0;
				j = strcspn(tmpVin->Commentaire + i, "");
				if (  j < 74 ) {
					sprintf(print_buffer,_("| %*.*s %*s |\n"),j,j,&tmpVin->Commentaire[i],73-j," ");
					i=i+j+1;
				}
				else {
					sprintf(print_buffer,_("| %74.74s |\n"),tmpVin->Commentaire+i);
					if (j == 74) i++;
					i=i+74;
				}
				write(fd,print_buffer,strlen(print_buffer)); 
				memset(print_buffer,'\0',sizeof(print_buffer));
			}
		}
	
		write(fd,print_ligne_separ,strlen(print_ligne_separ)); 
	 } /* endif Vin_rowSelected != NULL */
  } /* end else */

  close (fd);

  if ( flagprint == lpr ) {
    pid = fork ();

    if (pid > 0) { /* Processus pre      */
	  waitpid(pid,0,0);
    } 
    else 
       if (pid == 0) { /* Processus fils      */
         char * argv [5];
         int i=0,j=0,k=0;
         int cr=0;

	 /* on recupere la commande d'impression et les arguments dans la zone de 
	  * saisie 
	  */
         for (i = 0; i < strlen(tprinter) + 1; i++) {
            if ( (strncmp( tprinter + i , " "  , 1) == 0) || tprinter[i] == '\0' ) {
              argv[j] = ( char * ) malloc (i-k+1);
              memset(argv[j] , '\0' , i-k+1);
              strncpy(argv[j] , tprinter + k , i-k);
              k = i;
              j++;
            }
         }
       
         argv[j] = ( char * ) malloc (strlen(bprinter)+1);
         strncpy(argv[j] , bprinter , strlen(bprinter)+1);
	  
	 /* je ne sais pas le nombre d'argument, limitation a ... quelques uns
	  * pour l'instant.
	  */
         switch(j) {
	   case 0: g_warning("printer fatal error");
		   break;
	   case 1: cr=execlp (argv[0],argv[0],argv[1],NULL);
		   break;
	   case 2: cr=execlp (argv[0],argv[0],argv[1],argv[2],NULL);
		   break;
	   case 3: cr=execlp (argv[0],argv[0],argv[1],argv[2],argv[3],NULL);
		   break;
	   case 4: cr=execlp (argv[0],argv[0],argv[1],argv[2],argv[3],argv[4],NULL);
		   break;
	   default: g_warning("printer fatal error"); break;
         }
	 
         /* cr=execlp ("lpr","lpr","/root/xcave.txt",NULL); */
          
	 for (i=j;i>0;i--) free(argv[i]);
         _exit(0);
       } 
       else {
            /* Traitement d'erreur */
            g_warning ("fork fatal error");
            }
  }

  gtk_object_destroy (GTK_OBJECT (window_print));

   /* 
    * reactive window_xcave
    */
   gtk_widget_set_sensitive(window_xcave, TRUE);

#if (defined DEBUG_LEVEL_1)
	printf("on_window_print_button_ok_clicked:B:\n");
#endif
}


void
on_radiobutton_print_lpr_pressed(GtkButton       *button,
                                 gpointer         user_data)
{
#if (defined DEBUG_LEVEL_1)
	printf("on_radiobutton_print_lpr_pressed:B:\n");
#endif
 flagprint = lpr;
 gtk_entry_set_text(GTK_ENTRY(lookup_widget (window_print, "entry_printer")),"lpr");
#if (defined DEBUG_LEVEL_1)
	printf("on_radiobutton_print_lpr_pressed:E:\n");
#endif
}


void
on_radiobutton_print_file_pressed(GtkButton       *button,
                                  gpointer         user_data)
{
#if (defined DEBUG_LEVEL_1)
	printf("on_radiobutton_print_file_pressed:B:\n");
#endif
 flagprint = file;
 gtk_entry_set_text(GTK_ENTRY(lookup_widget (window_print, "entry_printer")),FileLocation);
#if (defined DEBUG_LEVEL_1)
	printf("on_radiobutton_print_file_pressed:E:\n");
#endif
}


void
on_radiobutton_print_this_pressed(GtkButton       *button,
                                  gpointer         user_data)
{
	Vin		*Vin_rowSelected;

#if (defined DEBUG_LEVEL_1)
	printf("on_radiobutton_print_this_pressed:B:\n");
#endif

	Vin_rowSelected = xcave_id_selected_treelist_row();
	if (Vin_rowSelected != NULL)
		flagrecord = record;
	else
		quick_message (_("No selected row"));

#if (defined DEBUG_LEVEL_1)
	printf("on_radiobutton_print_this_pressed:E:\n");
#endif

}

void
on_radiobutton_print_all_pressed(GtkButton       *button,
                                 gpointer         user_data)
{
#if (defined DEBUG_LEVEL_1)
	printf("on_radiobutton_print_all_pressed:B:\n");
#endif
	flagrecord = all;
#if (defined DEBUG_LEVEL_1)
	printf("on_radiobutton_print_all_pressed:E:\n");
#endif

}
